#include <iostream>
#include <cfloat>
#include <vector>

#include "myio.h"

using namespace std;

int main(int argc,char **argv){
    myio_init(argc,argv);

    int nrec,nt,nsou;
    float orec=0.,ot,osou=0.,drec,dt,dsou,zrec,zsou;

    from_header("data","n1",nrec,"d1",drec);
    from_header("data","n2",nt,"o2",ot,"d2",dt);
    from_header("data","n3",nsou,"d3",dsou);
    
    float *data=new float[nrec*nt*nsou];
    read("data",data,nrec*nt*nsou);
    
    vector<int> nrecPerShot;
    vector<float> recXLoc;
    vector<float> souXLoc;
    vector<float> ddata;
    
    int nsoutotal=0;

    for(int is=0;is<nsou;is++){
        int nr=0;
        for(int ir=0;ir<nrec;ir++){
            double amp=0.;
            for(int it=0;it<nt;it++) amp+=data[ir+it*nrec+is*nrec*nt]*data[ir+it*nrec+is*nrec*nt];
            if(amp>FLT_EPSILON){
                nr++;
                recXLoc.push_back(orec+ir*drec);
                for(int it=0;it<nt;it++) ddata.push_back(data[ir+it*nrec+is*nrec*nt]);
            }
        }
        cout<<"shot "<<is<<" has "<<nr<<" recs"<<endl;
        if(nr>0){
            nrecPerShot.push_back(nr);
            nsoutotal++;
            souXLoc.push_back(osou+is*dsou);
        }
    }

    get_param("zsou",zsou,"zrec",zrec);

    cout<<"total "<<nsoutotal<<" shot"<<endl;

    float *souloc=new float[nsoutotal*4]();
    int nrecsofar=0;
    for(int i=0;i<nsoutotal;i++){
     souloc[4*i]=souXLoc[i];
     souloc[4*i+1]=zsou;
     souloc[4*i+2]=nrecPerShot[i];
     souloc[4*i+3]=nrecsofar;
     nrecsofar+=nrecPerShot[i];
     cout<<"shot "<<i<<" has "<<souloc[4*i+2]<<" recs starting from "<<souloc[4*i+3]<<endl;
    }
    
    int nrectotal=recXLoc.size();
    float *recloc=new float[2*nrectotal]();
    #pragma omp parallel for num_threads(16)
    for(int i=0;i<nrectotal;i++){
     recloc[2*i]=recXLoc[i];
     recloc[2*i+1]=zrec;
    }
    
    to_header("souloc","n1",4,"o1",1.,"d1",1.);
    to_header("souloc","n2",nsoutotal,"o2",1.,"d2",1.);
    write("souloc",souloc,nsoutotal*4);
    
    to_header("recloc","n1",2,"o1",1.,"d1",1.);
    to_header("recloc","n2",nrectotal,"o2",1.,"d2",1.);
    write("recloc",recloc,2*nrectotal);

    if(nrectotal!=ddata.size()/nt) cout<<"sth is wrong"<<endl;

    write("ddata",&ddata[0],ddata.size());
    to_header("ddata","n1",nt,"o1",ot,"d1",dt);
    to_header("ddata","n2",nrectotal,"o2",1.,"d2",1.);
    
    delete []data;delete []souloc;delete []recloc;

    myio_close();

    return 0;
}
