#include "filter.h"
#include <iostream>

#include <Eigen/Cholesky>

using namespace std;
using namespace Eigen;

void shaping_filter(const float *original_data,const float *desired_data,int data_length,float *filter,int nNegLag,int nPosLag){
    int filter_length=1+nNegLag+nPosLag;
    
    MatrixXd A(data_length,filter_length);
    for(int j=0;j<filter_length;j++){
        for(int i=0;i<data_length;i++){
            int k=i-j+nNegLag;
            if(k>-1 && k<data_length){
                A(i,j)=original_data[k];
            }
            else A(i,j)=0.;
        }
    }
    VectorXd y(data_length);
    for(int i=0;i<data_length;i++) y(i)=desired_data[i];

    MatrixXd AtA=A.transpose()*A;
    VectorXd Aty=A.transpose()*y;
    VectorXd x=AtA.ldlt().solve(Aty);
    for(int i=0;i<filter_length;i++) filter[i]=x(i);
    
    return;
}

void shaping_filter(const float *original_data,const float *desired_data,int data_length1,int data_length2,float *filter,int nNegLag,int nPosLag){
    int filter_length=1+nNegLag+nPosLag;
    
    MatrixXd A(data_length1*data_length2,filter_length);
    for(int l=0;l<data_length2;l++){
        for(int j=0;j<filter_length;j++){
            for(int i=0;i<data_length1;i++){
                int k=i-j+nNegLag;
                if(k>-1 && k<data_length1){
                    A(l*data_length1+i,j)=original_data[k+l*data_length1];
                }
                else{
                    A(l*data_length1+i,j)=0.;
                }
            }
        
        }
    }
    VectorXd y(data_length1*data_length2);
    for(int i=0;i<data_length1*data_length2;i++) y(i)=desired_data[i];

    MatrixXd AtA=A.transpose()*A;
    VectorXd Aty=A.transpose()*y;
    VectorXd x=AtA.ldlt().solve(Aty);
    for(int i=0;i<filter_length;i++) filter[i]=x(i);
    
    return;
}

void conv(float *dataout,float *datain,int data_length,const float *filter,int nNegLag,int nPosLag,bool adj){
    if(adj){
        for(int i=0;i<data_length;i++){
            datain[i]=dataout[i]*filter[nNegLag];
            for(int j=0;j<nNegLag;j++){
                if(i-j>0){
                    datain[i]+=dataout[i-j-1]*filter[nNegLag-j-1];
                }
            }
            for(int j=0;j<nPosLag;j++){
                if(i+j<data_length-1){
                    datain[i]+=dataout[i+j+1]*filter[nNegLag+j+1];
                }
            }
        }
    }
    else{
        for(int i=0;i<data_length;i++){
            dataout[i]=datain[i]*filter[nNegLag];
            for(int j=0;j<nNegLag;j++){
                if(i+j<data_length-1){
                    dataout[i]+=datain[i+j+1]*filter[nNegLag-j-1];
                }
            }
            for(int j=0;j<nPosLag;j++){
                if(i-j>0){
                    dataout[i]+=datain[i-j-1]*filter[nNegLag+j+1];
                }
            }
        }
    }
    return;
}
