#include <cstdio>
#include "myio.h"
#include "mylib.h"
#include "init.h"
#include "solvers.h"

using namespace std;

int main(int argc,char **argv){
 myio_init(argc,argv);

 int nnx,nnz,nx,nz,nt,npad;
 float ox,oz,ot,dx,dz,dt;
 init2d(nnx,nnz,nx,nz,nt,dx,dz,dt,ox,oz,ot,npad);

 loat *wavelet=new float[nt];
 read("wavelet",wavelet,nt);
 
 int ns;
 from_header("souloc","n2",ns);
 float *souloc=new float[ns*4]();
 read("souloc",souloc,ns*4);
 int *sloc=new int[ns*4]();
 #pragma omp parallel for num_threads(16)
 for(int is=0;is<ns;is++){
  sloc[is*4+0]=(souloc[is*4+0]-ox)/dx+0.5+npad;
  sloc[is*4+1]=(souloc[is*4+1]-oz)/dz+0.5+npad;
  sloc[is*4+2]=souloc[is*4+2];
  sloc[is*4+3]=souloc[is*4+3];
 }

 int nr;
 from_header("recloc","n2",nr);
 float *recloc=new float[nr*2]();
 read("recloc",recloc,nr*2);
 int *rloc=new int[nr*2]();
 #pragma omp parallel for num_threads(16)
 for(int ir=0;ir<nr;ir++){
  rloc[ir*2+0]=(recloc[ir*2+0]-ox)/dx+0.5+npad;
  rloc[ir*2+1]=(recloc[ir*2+1]-oz)/dz+0.5+npad;
 }

 float rate;
 get_param("rate",rate);
 int ratio=rate/dt+0.5;
 int nnt=(nt-1)/ratio+1;
 float *data=new float[nr*nnt]();
 read("data",data,nr*nnt);
 
 float *taper=new float[nnx*nnz]();
 init_abc(taper,nx,nz,npad);

 float wbottom; get_param("wbottom",wbottom);

 int niter; get_param("niter",niter);

 float a0; get_param("a0",a0);
 fprintf(stderr,"Starting step length %.10f\n",a0);
 
 int padded; get_param("padded",padded);
 
  float *v=new float[nnx*nnz]();
  float *eps=new float[nnx*nnz]();
  float *del=new float[nnx*nnz]();
  
  if(padded==0){
   init_model("v",v,nx,nz,npad); 
   init_model("eps",eps,nx,nz,npad); 
   init_model("del",del,nx,nz,npad); 
  } 
  else{ 
   read("v",v,nnx*nnz);
   read("eps",eps,nnx*nnz);
   read("del",del,nnx*nnz);
  }
  
  float v0,eps0,del0;
  get_param("v0",v0,"eps0",eps0,"del0",del0);
 
  scale(v,v,1./v0,nnx*nnz);
  scale(eps,eps,1./eps0,nnx*nnz);
  scale(del,del,1./del0,nnx*nnz);

  float randfactor;
  get_param("randfactor",randfactor);

   adam1(v,eps,del,v0,eps0,del0,niter,a0,data,wavelet,sloc,ns,rloc,nr,taper,nx,nz,nt,npad,dx,dz,dt,rate,wbottom,randfactor);

  fprintf(stderr,"Done \n");
  delete []v;delete []eps;delete []del;
 
 delete []wavelet;delete []data;delete []sloc;delete []rloc;delete []taper;
 delete []souloc; delete []recloc;

 myio_close();
 return 0;
}
