#include <cstdio>
#include <string>
#include <cstring>

#include "myio.h"
#include "mylib.h"
#include "init.h"
#include "wave.h"

using namespace std;

extern "C"{
 void setulb_(int *n,int *m,float *x,float *l,float *u,int *nbd,float *f,float *g,float *factr,float *pgtol,float *wa,int *iwa,char task[],int *iprint,char csave[],int lsave[],int isave[],float dsave[],unsigned int tasklen,unsigned int csavelen);
}

int main(int argc,char **argv){
 myio_init(argc,argv);

 int nnx,nnz,nx,nz,nt,npad;
 float ox,oz,ot,dx,dz,dt;
 init2d(nnx,nnz,nx,nz,nt,dx,dz,dt,ox,oz,ot,npad);

 float *wavelet=new float[nt];
 read("wavelet",wavelet,nt);
 
 int ns;
 from_header("souloc","n2",ns);
 float *souloc=new float[ns*4]();
 read("souloc",souloc,ns*4);
 int *sloc=new int[ns*4]();
 #pragma omp parallel for num_threads(16)
 for(int is=0;is<ns;is++){
  sloc[is*4+0]=(souloc[is*4+0]-ox)/dx+0.5+npad;
  sloc[is*4+1]=(souloc[is*4+1]-oz)/dz+0.5+npad;
  sloc[is*4+2]=souloc[is*4+2];
  sloc[is*4+3]=souloc[is*4+3];
 }

 int nr;
 from_header("recloc","n2",nr);
 float *recloc=new float[nr*2]();
 read("recloc",recloc,nr*2);
 int *rloc=new int[nr*2]();
 #pragma omp parallel for num_threads(16)
 for(int ir=0;ir<nr;ir++){
  rloc[ir*2+0]=(recloc[ir*2+0]-ox)/dx+0.5+npad;
  rloc[ir*2+1]=(recloc[ir*2+1]-oz)/dz+0.5+npad;
 }

 float rate;
 get_param("rate",rate);
 int ratio=rate/dt+0.5;
 int ntNeg=std::round(abs(ot)/dt);
 int nnt_data=(nt-ntNeg-1)/ratio+1;
 float *d0=new float[nr*nnt_data]();
 read("data",d0,nr*nnt_data);
 
 float *taper=new float[nnx*nnz]();
 init_abc(taper,nx,nz,npad);

 float wbottom; get_param("wbottom",wbottom);

 int padded; get_param("padded",padded);
 
 float *m=new float[nnx*nnz]();
 
 float *vnetadel=new float[3*nnx*nnz]();
 float *vn=vnetadel,*eta=vnetadel+nnx*nnz,*del=vnetadel+2*nnx*nnz;

 if(padded==0){
  init_model("vn",vn,nx,nz,npad); 
  init_model("eta",eta,nx,nz,npad); 
  init_model("del",del,nx,nz,npad); 
 } 
 else{ 
  read("vn",vn,nnx*nnz);
  read("eta",eta,nnx*nnz);
  read("del",del,nnx*nnz);
 }
 
 float vn0,eta0,del0;
 get_param("vn0",vn0,"eta0",eta0,"del0",del0);

 scale(vn,vn,1./vn0,nnx*nnz);
 scale(eta,eta,1./eta0,nnx*nnz);
 scale(del,del,1./del0,nnx*nnz);

 float *gvnetadel=new float[3*nnx*nnz]();
 float *gvn=gvnetadel,*geta=gvnetadel+nnx*nnz,*gdel=gvnetadel+2*nnx*nnz;

 int nfg; get_param("nfg",nfg);

 int nn=3*nnx*nnz,mm=5,iprint=-1,lsave[4],isave[44];
 float factr=0.,pgtol=0.,f,dsave[29];
 char task[60],csave[60];
 int *nbd=new int[nn]();
 set(nbd,2,nn);
 int *iwa=new int[3*nn]();
 float *l=new float[nn]();
 float *u=new float[nn]();
 float *wa=new float[2*mm*nn+5*nn+11*mm*mm+8*mm]();

 float *lvn=l,*leta=l+nnx*nnz,*ldel=l+2*nnx*nnz;
 float *uvn=u,*ueta=u+nnx*nnz,*udel=u+2*nnx*nnz;

 if(padded==0){
  init_model("lvn",lvn,nx,nz,npad); 
  init_model("leta",leta,nx,nz,npad); 
  init_model("ldel",ldel,nx,nz,npad); 
  init_model("uvn",uvn,nx,nz,npad); 
  init_model("ueta",ueta,nx,nz,npad); 
  init_model("udel",udel,nx,nz,npad); 
 } 
 else{ 
  read("lvn",lvn,nnx*nnz);
  read("leta",leta,nnx*nnz);
  read("ldel",ldel,nnx*nnz);
  read("uvn",uvn,nnx*nnz);
  read("ueta",ueta,nnx*nnz);
  read("udel",udel,nnx*nnz);
 }
 
 scale(lvn,lvn,1./vn0,nnx*nnz);
 scale(leta,leta,1./eta0,nnx*nnz);
 scale(ldel,ldel,1./del0,nnx*nnz);

 scale(uvn,uvn,1./vn0,nnx*nnz);
 scale(ueta,ueta,1./eta0,nnx*nnz);
 scale(udel,udel,1./del0,nnx*nnz);

 strcpy(task,"START");
 for(int i=5;i<60;++i) task[i]=' ';

 int nnew=0;

 while((task[0]=='F' && task[1]=='G') || 
       (task[0]=='N' && task[1]=='E' && task[2]=='W' && task[3]=='_' && task[4]=='X') || 
       (task[0]=='S' && task[1]=='T' && task[2]=='A' && task[3]=='R' && task[4]=='T')){
  
  setulb_(&nn,&mm,vnetadel,l,u,nbd,&f,gvnetadel,&factr,&pgtol,wa,iwa,task,&iprint,csave,lsave,isave,dsave,60,60);

  if(task[0]=='F' && task[1]=='G'){
   f=objFuncGradient2(gvn,geta,gdel,d0,vn,eta,del,vn0,eta0,del0,wavelet,sloc,ns,rloc,nr,taper,nx,nz,nt,npad,dx,dz,dt,rate,ot,wbottom,m);
  }
  else{
   if(task[0]=='N' && task[1]=='E' && task[2]=='W' && task[3]=='_' && task[4]=='X'){
	nnew++;

    fprintf(stderr,"New x %d iterate %d nfg=%d f=%.10f |proj g|=%.10f\n",nnew,isave[29],isave[33],f,dsave[12]);
    
    write("objfunc",&f,1,"native_float",std::ios_base::app);
    write("ivn",vnetadel,nnx*nnz,"native_float",std::ios_base::app);
    write("ieta",vnetadel+nnx*nnz,nnx*nnz,"native_float",std::ios_base::app);
    write("idel",vnetadel+2*nnx*nnz,nnx*nnz,"native_float",std::ios_base::app);
 
	if(isave[33]>=nfg){
	 fprintf(stderr,"STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT");
	 break; 
	}
   
    if(dsave[12]<=1e-10*(1+fabs(f))){
	 fprintf(stderr,"STOP: THE PROJECTED GRADIENT IS SUFFICIENTLY SMALL");
	 break; 
	}
   }
  }
 }

 to_header("objfunc","n1",nnew,"o1",0.,"d1",1.);
 to_header("ivn","n1",nnx,"o1",-dx*npad,"d1",dx);
 to_header("ivn","n2",nnz,"o2",-dz*npad,"d2",dz);
 to_header("ivn","n3",nnew,"o3",0.,"d3",1.);
 to_header("ieta","n1",nnx,"o1",-dx*npad,"d1",dx);
 to_header("ieta","n2",nnz,"o2",-dz*npad,"d2",dz);
 to_header("ieta","n3",nnew,"o3",0.,"d3",1.);
 to_header("idel","n1",nnx,"o1",-dx*npad,"d1",dx);
 to_header("idel","n2",nnz,"o2",-dz*npad,"d2",dz);
 to_header("idel","n3",nnew,"o3",0.,"d3",1.);
 
 delete []vnetadel;delete []gvnetadel;
 delete []nbd;delete []iwa;delete []l;delete []u;delete []wa;
 delete []wavelet;delete []d0;delete []sloc;delete []rloc;delete []taper;
 delete []souloc; delete []recloc;

 myio_close();
 return 0;
}
