#include <cmath>
#include <omp.h>
#include <cstdio>

#include "myio.h"
#include "mylib.h"
#include "init.h"
#include "wave.h"

using namespace std;

int main(int argc,char **argc11){
 myio_init(argc,argc11);

int nnx,nnz,nx,nz,nt,npad;
 float ox,oz,ot,dx,dz,dt;
 init2d(nnx,nnz,nx,nz,nt,dx,dz,dt,ox,oz,ot,npad);

 float *wavelet=new float[nt];
 read("wavelet",wavelet,nt);
 
 int ns;
 from_header("souloc","n2",ns);
 float *souloc=new float[ns*4]();
 read("souloc",souloc,ns*4);
 int *sloc=new int[ns*4]();
 #pragma omp parallel for num_threads(16)
 for(int is=0;is<ns;is++){
  sloc[is*4+0]=(souloc[is*4+0]-ox)/dx+0.5+npad;
  sloc[is*4+1]=(souloc[is*4+1]-oz)/dz+0.5+npad;
  sloc[is*4+2]=souloc[is*4+2];
  sloc[is*4+3]=souloc[is*4+3];
 }

 int nr;
 from_header("recloc","n2",nr);
 float *recloc=new float[nr*2]();
 read("recloc",recloc,nr*2);
 int *rloc=new int[nr*2]();
 #pragma omp parallel for num_threads(16)
 for(int ir=0;ir<nr;ir++){
  rloc[ir*2+0]=(recloc[ir*2+0]-ox)/dx+0.5+npad;
  rloc[ir*2+1]=(recloc[ir*2+1]-oz)/dz+0.5+npad;
 }

 float rate;
 get_param("rate",rate);
 int ratio=rate/dt+0.5;
 int ntNeg=std::round(abs(ot)/dt);
 int nnt_data=(nt-ntNeg-1)/ratio+1;
 float *data=new float[nr*nnt_data]();
 read("data",data,nr*nnt_data);

 float *taper=new float[nnx*nnz]();
 init_abc(taper,nx,nz,npad);
 
 float wbottom; get_param("wbottom",wbottom);

 int padded; get_param("padded",padded);
 
 float *m=new float[nnx*nnz]();

  float *v=new float[nnx*nnz]();
  float *eps=new float[nnx*nnz]();
  float *del=new float[nnx*nnz]();
  
  if(padded==0){
   init_model("v",v,nx,nz,npad); 
   init_model("eps",eps,nx,nz,npad); 
   init_model("del",del,nx,nz,npad); 
  } 
  else{ 
   read("v",v,nnx*nnz);
   read("eps",eps,nnx*nnz);
   read("del",del,nnx*nnz);
  }
 
  float v0,eps0,del0;
  get_param("v0",v0,"eps0",eps0,"del0",del0);
 
  scale(v,v,1./v0,nnx*nnz);
  scale(eps,eps,1./eps0,nnx*nnz);
  scale(del,del,1./del0,nnx*nnz);
  
  float *gv=new float[nnx*nnz]();
  float *geps=new float[nnx*nnz]();
  float *gdel=new float[nnx*nnz]();
  
   float val=objFuncGradientVEpsDel(gv,geps,gdel,data,v,eps,del,v0,eps0,del0,wavelet,sloc,ns,rloc,nr,taper,nx,nz,nt,npad,dx,dz,dt,rate,ot,wbottom,m);
   cout<<"Objfunc "<<val<<endl;
 
  write("gv",gv,nnx*nnz);
  to_header("gv","n1",nnx,"o1",-dx*npad,"d1",dx);
  to_header("gv","n2",nnz,"o2",-dz*npad,"d2",dz);
  write("geps",geps,nnx*nnz);
  to_header("geps","n1",nnx,"o1",-dx*npad,"d1",dx);
  to_header("geps","n2",nnz,"o2",-dz*npad,"d2",dz);
  write("gdel",gdel,nnx*nnz);
  to_header("gdel","n1",nnx,"o1",-dx*npad,"d1",dx);
  to_header("gdel","n2",nnz,"o2",-dz*npad,"d2",dz);
 
  delete []v; delete []eps; delete []del;
  delete []gv; delete []geps; delete []gdel;
 
 delete []wavelet;delete []data;delete []sloc;delete []rloc;delete []taper;
 delete []m;
 delete []souloc;delete []recloc;

 myio_close();
 return 0;
}
