#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <iostream>
#include <cfloat>

#include "myio.h"
#include "mylib.h"
#include "processing.h"

using namespace std;

int main(int argc,char **argv){
    myio_init(argc,argv);

    int nt,ncmp,noff;
    float ot,dt,ocmp,dcmp,ooff,doff;
    from_header("cmp_gather","n1",nt,"o1",ot,"d1",dt);
    from_header("cmp_gather","n2",noff,"o2",ooff,"d2",doff);
    from_header("cmp_gather","n3",ncmp,"o3",ocmp,"d3",dcmp);
    float *gather=new float[nt*noff*ncmp];
    read("cmp_gather",gather,nt*noff*ncmp);
    
    float *vrms=new float[nt*ncmp];
    read("vrms",vrms,nt*ncmp);

    float mineta,maxeta,deta;
    get_param("mineta",mineta,"maxeta",maxeta,"deta",deta);
    int neta=(maxeta-mineta)/deta+1;

    float *dataout=new float[nt*noff*neta]();
    float *eta=new float[nt]();
   
    float *offset=new float[noff];
    for(int ioff=0;ioff<noff;ioff++) offset[ioff]=ioff*doff+ooff;
    
    int halfwindow;
    get_param("halfwindow",halfwindow);

    float *sem=new float[nt*neta*ncmp]();
    for(int icmp=0;icmp<ncmp;icmp++){
        fprintf(stderr,"nmo for cmp %d\n",icmp);
        for(int ieta=0;ieta<neta;ieta++){
            set(eta,mineta+ieta*deta,nt);
            aniso_nmo(dataout+ieta*nt*noff,gather+icmp*nt*noff,vrms+icmp*nt,eta,nt,ot,dt,offset,noff);
            for(int ioff=0;ioff<noff;ioff++){
                int nzero=(offset[ioff]/3500.-ot)/dt+1;
                memset(dataout+ioff*nt+ieta*nt*noff,0,nzero*sizeof(float));
            }
        }
        write("dataout",dataout,nt*noff*neta,"native_float",std::ios_base::app);
        fprintf(stderr,"sem for cmp %d\n",icmp);
        for(int ieta=0;ieta<neta;ieta++){
            for(int it=halfwindow;it<nt-halfwindow;it++){
                double a=0.,b=0.;
                for(int i=it-halfwindow;i<=it+halfwindow;i++){
                    double temp=0.;
                    for(int ioff=0;ioff<noff;ioff++){
                        double c=dataout[i+ioff*nt+ieta*nt*noff];
                        temp+=c;
                        b+=c*c;
                    }
                    a+=temp*temp;
                }
                b+=FLT_EPSILON;
                sem[it+ieta*nt+icmp*nt*neta]=a/b/noff;
            }
        }
    }
    
    to_header("dataout","n1",nt,"o1",0.,"d1",dt);
    to_header("dataout","n2",noff,"o2",ooff,"d2",doff);
    to_header("dataout","n3",neta,"o3",mineta,"d3",deta);
    to_header("dataout","n4",ncmp,"o4",ocmp,"d4",dcmp);

    write("semblance",sem,nt*neta*ncmp);
    to_header("semblance","n1",nt,"o1",0.,"d1",dt);
    to_header("semblance","n2",neta,"o2",mineta,"d2",deta);
    to_header("semblance","n3",ncmp,"o3",ocmp,"d3",dcmp);

    delete gather;
    delete []vrms;
    delete []dataout;
    delete []sem;
    delete []offset;
    delete []eta;

    myio_close();
    return 0;
}
