#!/usr/bin/env python

import numpy as np
from seppyio import *
import matplotlib
import matplotlib.pyplot as plt

matplotlib.rcParams.update({'font.size': 15})

nx,ny=from_header("vfile","n1","n2")
ox,dx=from_header("vfile","o1","d1")
oy,dy=from_header("vfile","o2","d2")
dx=dx*1e-3
dy=dy*1e-3

wellx=np.array([650796.7716,662940.6096,651775.7857,683131.4882,679538.8049,669467.0754])-ox
wellx=wellx*1e-3
welly=np.array([1048.853339,-4868.383954,-8624.902802,-22954.51988,-6968.619542,-10249.09604])-oy
welly=welly*1e-3
wellname=['SS160','SS187','SS191','ST200','ST143','ST168']

v=read("vfile",(ny,nx))
v=v*1e-3
minv=1.5
maxv=4.5

fig, ax = plt.subplots(figsize=(10,8))
cax=ax.imshow(v,interpolation='none',aspect='auto',extent=[0,(nx-1)*dx,(ny-1)*dy,0],cmap='jet',vmin=minv,vmax=maxv)
ax.scatter(wellx,welly,s=100,marker='*',c='k')

for i,txt in enumerate(wellname):
    ax.annotate(txt,xy=(wellx[i],welly[i]),xytext=(wellx[i]-1.5,welly[i]+0.5),color='black')

dv=maxv-minv
plt.xlabel('X (km)')
plt.ylabel('Y (km)')
cbar=fig.colorbar(cax,ticks=[minv,minv+dv/5,minv+2*dv/5,minv+3*dv/5,minv+4*dv/5,maxv])
cbar.ax.set_ylabel('Velocity (km/s)')
plt.gca().invert_yaxis()

#plt.show()
plt.savefig('../Fig/welllocation0.pdf',bbox_inches='tight')

x=np.linspace(0,1066*25,1067)
x=x*1e-3
y=np.linspace(-10320-oy,-10320-oy,1067)
y=y*1e-3
plt.plot(x,y,linewidth=3,linestyle='--',c='k')

#plt.show()
plt.savefig('../Fig/welllocation1.pdf',bbox_inches='tight')

#plt.show()
